@@ -30,6 +30,7 @@ extern "C" {
 #include <glib/gi18n.h>
 #include <pan/general/debug.h>
 #include <pan/general/e-util.h>
+#include <pan/general/editor-spawner.h>
 #include <pan/general/file-util.h>
 #include <pan/general/macros.h>
 #include <pan/usenet-utils/scorefile.h>
@@ -54,11 +55,7 @@ extern "C" {
 #include "gui.h"
 #include "pad.h"
 
-#ifdef HAVE_GNUTLS
-  #include "pan.ui.ssl.h"
-#else
-  #include "pan.ui.h"
-#endif
+#include "pan.ui.h"
 
 #include "prefs-ui.h"
 #include "progress-view.h"
@@ -77,6 +74,8 @@ extern "C" {
 
 #include <pan/usenet-utils/gpg.h>
 
+#include <pan/general/null.h>
+
 namespace
 {
   std::string get_accel_filename () {
@@ -98,7 +97,7 @@ namespace pan
   void remove_from_parent (GtkWidget * w)
   {
     GtkWidget *parent = gtk_widget_get_parent(w);
-    if (parent != 0)
+    if (parent != nullptr)
       gtk_container_remove (GTK_CONTAINER(parent), w);
   }
 }
@@ -215,24 +214,24 @@ GUI :: root_realized_cb (GtkWidget*, gpointer self_gpointer)
 GUI :: GUI (Data& data, Queue& queue, Prefs& prefs, GroupPrefs& group_prefs):
   _data (data),
   _queue (queue),
-  _prefs (prefs),
-  _group_prefs (group_prefs),
   _cache (data.get_cache()),
   _encode_cache (data.get_encode_cache()),
+  _prefs (prefs),
+  _group_prefs (group_prefs),
+  _certstore(data.get_certstore()),
   _root (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0)),
   _menu_vbox (gtk_box_new (GTK_ORIENTATION_VERTICAL, 0)),
-  _group_pane (0),
-  _header_pane (0),
-  _body_pane (0),
+  _group_pane (nullptr),
+  _header_pane (nullptr),
+  _body_pane (nullptr),
   _ui_manager (gtk_ui_manager_new ()),
   _info_image (gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_MENU)),
   _error_image (gtk_image_new_from_stock (GTK_STOCK_DIALOG_ERROR, GTK_ICON_SIZE_MENU)),
-  _connection_size_eventbox (0),
-  _connection_size_label (0),
-  _queue_size_label (0),
-  _queue_size_button (0),
-  _taskbar (0),
-  _certstore(data.get_certstore())
+  _connection_size_eventbox (nullptr),
+  _connection_size_label (nullptr),
+  _queue_size_label (nullptr),
+  _queue_size_button (nullptr),
+  _taskbar (nullptr)
 {
 
   char * filename = g_build_filename (file::get_pan_home().c_str(), "pan.ui", NULL);
@@ -394,9 +393,9 @@ GUI :: GUI (Data& data, Queue& queue, Prefs& prefs, GroupPrefs& group_prefs):
 
 namespace
 {
-  GtkWidget * hpane (0);
-  GtkWidget * vpane (0);
-  GtkWidget * sep_vpane (0);
+  GtkWidget * hpane (nullptr);
+  GtkWidget * vpane (nullptr);
+  GtkWidget * sep_vpane (nullptr);
 }
 
 GUI :: ~GUI ()
@@ -495,7 +494,7 @@ namespace
     if (!key_to_action.empty())
       return;
 
-    for (GList * l=gtk_ui_manager_get_action_groups(uim); l!=0; l=l->next)
+    for (GList * l=gtk_ui_manager_get_action_groups(uim); l!=nullptr; l=l->next)
     {
       GtkActionGroup * action_group = GTK_ACTION_GROUP(l->data);
       GList * actions = gtk_action_group_list_actions (action_group);
@@ -704,7 +703,7 @@ void GUI :: do_save_articles_to_nzb ()
       Queue::tasks_t tasks;
       std::string emptystring;
       foreach_const (std::vector<Article>, copies, it)
-        tasks.push_back (new TaskArticle (_data, _data, *it, _cache, _data, always ? TaskArticle::ALWAYS_MARK : TaskArticle::NEVER_MARK, 0, TaskArticle::RAW,emptystring));
+        tasks.push_back (new TaskArticle (_data, _data, *it, _cache, _data, always ? TaskArticle::ALWAYS_MARK : TaskArticle::NEVER_MARK, nullptr, TaskArticle::RAW,emptystring));
 
       // write them to a file
       std::ofstream tmp(file.c_str());
@@ -745,11 +744,7 @@ namespace
       if (GMIME_IS_PART(o))
       {
         GMimePart * part (GMIME_PART (o));
-#ifdef HAVE_GMIME_30
         GMimeDataWrapper * wrapper (g_mime_part_get_content (part));
-#else        
-        GMimeDataWrapper * wrapper (g_mime_part_get_content_object (part));
-#endif        
         GMimeStream * mem_stream (g_mime_stream_mem_new ());
         g_mime_data_wrapper_write_to_stream (wrapper, mem_stream);
         const GByteArray * buffer (GMIME_STREAM_MEM(mem_stream)->buffer);
@@ -764,7 +759,7 @@ namespace
 
     virtual ~SaveArticlesFromNZB() {}
 
-    virtual void on_progress_finished (Progress&, int status)
+    void on_progress_finished (Progress&, int status) override
     {
       if (status == OK) {
 #ifdef HAVE_GMIME_CRYPTO
@@ -831,7 +826,7 @@ void GUI :: do_import_tasks ()
   strings_t filenames;
   if (gtk_dialog_run (GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT) {
     GSList * tmp = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER(dialog));
-    for (GSList * l(tmp); l!=0; l=l->next) {
+    for (GSList * l(tmp); l!=nullptr; l=l->next) {
       filenames.push_back ((char*) l->data);
       g_free (l->data);
     }
@@ -869,13 +864,13 @@ namespace
   void task_pane_destroyed_cb (GtkWidget *, gpointer p)
   {
     TaskPane ** task_pane (static_cast<TaskPane**>(p));
-    *task_pane = 0;
+    *task_pane = nullptr;
   }
 }
 
 void GUI :: do_show_task_window ()
 {
-  static TaskPane * task_pane (0);
+  static TaskPane * task_pane (nullptr);
 
   if (!task_pane) {
     task_pane = new TaskPane (_queue, _prefs);
@@ -920,7 +915,7 @@ void GUI :: do_show_log_window ()
 {
   set_bin_child (_event_log_button, _info_image);
 
-  static GtkWidget * w (0);
+  static GtkWidget * w (nullptr);
   if (!w) {
     w = log_dialog_new (_prefs, get_window (_root));
     g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
@@ -1187,7 +1182,7 @@ void GUI :: do_expand_thread ()
 void GUI :: do_show_score_dialog ()
 {
   const Quark& group (_header_pane->get_group());
-  const Article * article (0);
+  const Article * article (nullptr);
   if (!group.empty())
     article = _header_pane->get_first_selected_article ();
   if (article) {
@@ -1312,7 +1307,7 @@ GUI :: score_add (int mode)
 
   Article a;
   const Article* article (_header_pane->get_first_selected_article ());
-  if (article != 0)
+  if (article != nullptr)
     a = *article;
 
   ScoreAddDialog * d = new ScoreAddDialog (_data, _root, group, a, (ScoreAddDialog::Mode)mode);
@@ -1326,12 +1321,8 @@ void GUI :: do_supersede_article ()
     return;
 
   // did this user post the message?
-#ifdef HAVE_GMIME_30
   InternetAddressList * Lsender (g_mime_message_get_sender (message));
   char * sender = internet_address_list_to_string(Lsender, NULL,  TRUE);
-#else  
-  const char * sender (g_mime_message_get_sender (message));
-#endif  
   const bool user_posted_this (_data.has_from_header (sender));
 
   if (!user_posted_this) {
@@ -1357,7 +1348,6 @@ void GUI :: do_supersede_article ()
   GMimeMessage * new_message (g_mime_message_new (false));
   GMimeObject * new_message_obj = (GMimeObject*)new_message;
 
-#ifdef HAVE_GMIME_30
   g_mime_object_set_header (new_message_obj, "Supersedes", old_mid, NULL);
   const char * addr = internet_address_list_to_string (g_mime_message_get_sender (message), NULL, TRUE);
   g_mime_message_add_mailbox (new_message, GMIME_ADDRESS_TYPE_SENDER, NULL, addr);
@@ -1369,34 +1359,19 @@ void GUI :: do_supersede_article ()
               g_mime_message_add_mailbox (new_message, GMIME_ADDRESS_TYPE_REPLY_TO, NULL, cpch);
   if ((cpch = g_mime_object_get_header ((GMimeObject *)message,     "Followup-To")))
               g_mime_object_set_header (new_message_obj, "Followup-To", cpch, NULL);
-#else  
-  g_mime_object_set_header (new_message_obj, "Supersedes", old_mid);
-  g_mime_message_set_sender (new_message, g_mime_message_get_sender (message));
-  g_mime_message_set_subject (new_message, g_mime_message_get_subject (message));
-  g_mime_object_set_header (new_message_obj, "Newsgroups", g_mime_object_get_header ((GMimeObject *)message, "Newsgroups"));
-  g_mime_object_set_header (new_message_obj, "References", g_mime_object_get_header ((GMimeObject *)message, "References"));
-  if ((cpch = g_mime_message_get_reply_to (message)))
-              g_mime_message_set_reply_to (new_message, cpch);
-  if ((cpch = g_mime_object_get_header ((GMimeObject *)message,     "Followup-To")))
-              g_mime_object_set_header (new_message_obj, "Followup-To", cpch);
-#endif  
   gboolean  unused (false);
   char * body (pan_g_mime_message_get_body (message, &unused));
   GMimeStream * stream = g_mime_stream_mem_new_with_buffer (body, strlen(body));
   g_free (body);
   GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
   GMimePart * part = g_mime_part_new ();
-#ifdef HAVE_GMIME_30
    g_mime_part_set_content(part, content_object);
-#else  
-  g_mime_part_set_content_object (part, content_object);
-#endif  
   g_mime_message_set_mime_part (new_message, GMIME_OBJECT(part));
   g_object_unref (part);
   g_object_unref (content_object);
   g_object_unref (stream);
 
-  PostUI * post = PostUI :: create_window (0, _data, _queue, _data, _data, new_message, _prefs, _group_prefs, _encode_cache);
+  PostUI * post = PostUI :: create_window (nullptr, _data, _queue, _data, _data, new_message, _prefs, _group_prefs, _encode_cache);
   if (post)
   {
     gtk_widget_show_all (post->root());
@@ -1424,11 +1399,7 @@ void GUI :: do_cancel_article ()
     return;
 
   // did this user post the message?
-#ifdef HAVE_GMIME_30
   const char * sender = internet_address_list_to_string(g_mime_message_get_sender (message), NULL, TRUE);
-#else  
-  const char * sender (g_mime_message_get_sender (message));
-#endif  
   const bool user_posted_this (_data.has_from_header (sender));
   if (!user_posted_this) {
     GtkWidget * w = gtk_message_dialog_new (
@@ -1448,35 +1419,23 @@ void GUI :: do_cancel_article ()
   // okay then...
   GMimeMessage * cancel = g_mime_message_new (false);
   char * cancel_message = g_strdup_printf ("cancel <%s>", g_mime_message_get_message_id(message));
-#ifdef HAVE_GMIME_30
-  const char * s_addr = internet_address_list_to_string(g_mime_message_get_sender (message), NULL, TRUE);
-  g_mime_message_add_mailbox (cancel, GMIME_ADDRESS_TYPE_SENDER, NULL, s_addr);
+  const char * sk_addr = internet_address_list_to_string(g_mime_message_get_sender (message), NULL, TRUE);
+  g_mime_message_add_mailbox (cancel, GMIME_ADDRESS_TYPE_SENDER, NULL, sk_addr);
   g_mime_message_set_subject (cancel, "Cancel", NULL);
   g_mime_object_set_header ((GMimeObject *)cancel, "Newsgroups", g_mime_object_get_header ((GMimeObject *)message, "Newsgroups"), NULL);
   g_mime_object_set_header ((GMimeObject *)cancel, "Control", cancel_message, NULL);
-#else  
-  g_mime_message_set_sender (cancel, g_mime_message_get_sender (message));
-  g_mime_message_set_subject (cancel, "Cancel");
-  g_mime_object_set_header ((GMimeObject *)cancel, "Newsgroups", g_mime_object_get_header ((GMimeObject *)message, "Newsgroups"));
-  g_mime_object_set_header ((GMimeObject *)cancel, "Control", cancel_message);
-
-#endif  
   const char * body ("Ignore\r\nArticle canceled by author using " PACKAGE_STRING "\r\n");
   GMimeStream * stream = g_mime_stream_mem_new_with_buffer (body, strlen(body));
   GMimeDataWrapper * content_object = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_DEFAULT);
   GMimePart * part = g_mime_part_new ();
-#ifdef HAVE_GMIME_30
   g_mime_part_set_content (part, content_object);
-#else  
-  g_mime_part_set_content_object (part, content_object);
-#endif  
   g_mime_message_set_mime_part (cancel, GMIME_OBJECT(part));
   g_object_unref (part);
   g_object_unref (content_object);
   g_object_unref (stream);
   g_free (cancel_message);
 
-  PostUI * post = PostUI :: create_window (0, _data, _queue, _data, _data, cancel, _prefs, _group_prefs, _encode_cache);
+  PostUI * post = PostUI :: create_window (nullptr, _data, _queue, _data, _data, cancel, _prefs, _group_prefs, _encode_cache);
   if (post)
   {
     gtk_widget_show_all (post->root());
@@ -1496,7 +1455,7 @@ void GUI :: do_cancel_article ()
   g_object_unref (message);
 }
 
-bool GUI::deletion_confirmation_dialog()
+bool GUI::deletion_confirmation_dialog() const
 {
   bool ret(false);
   GtkWidget * d = gtk_message_dialog_new (
@@ -1631,26 +1590,18 @@ GUI :: do_post ()
       newsgroups = group;
   }
   if (!newsgroups.empty())
-#ifdef HAVE_GMIME_30
     g_mime_object_append_header ((GMimeObject *) message, "Newsgroups", newsgroups.c_str(), NULL);
-#else    
-    g_mime_object_append_header ((GMimeObject *) message, "Newsgroups", newsgroups.c_str());
-#endif    
 
   // content type
   GMimePart * part = g_mime_part_new ();
-#ifdef HAVE_GMIME_30
   GMimeContentType *type = g_mime_content_type_parse (NULL, "text/plain; charset=UTF-8");
-#else  
-  GMimeContentType *type = g_mime_content_type_new_from_string ("text/plain; charset=UTF-8");
-#endif  
   g_mime_object_set_content_type ((GMimeObject *) part, type);
   g_object_unref (type);
   g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_8BIT);
   g_mime_message_set_mime_part (message, GMIME_OBJECT(part));
   g_object_unref (part);
 
-  PostUI * post = PostUI :: create_window (0, _data, _queue, _data, _data, message, _prefs, _group_prefs, _encode_cache);
+  PostUI * post = PostUI :: create_window (nullptr, _data, _queue, _data, _data, message, _prefs, _group_prefs, _encode_cache);
   if (post)
     gtk_widget_show_all (post->root());
   g_object_unref (message);
@@ -1660,7 +1611,7 @@ void GUI :: do_followup_to ()
 {
   GMimeMessage * message = _body_pane->create_followup_or_reply (false);
   if (message) {
-    PostUI * post = PostUI :: create_window(0, _data, _queue, _data, _data, message, _prefs, _group_prefs, _encode_cache);
+    PostUI * post = PostUI :: create_window(nullptr, _data, _queue, _data, _data, message, _prefs, _group_prefs, _encode_cache);
     if (post)
       gtk_widget_show_all (post->root());
     g_object_unref (message);
@@ -1670,7 +1621,7 @@ void GUI :: do_reply_to ()
 {
   GMimeMessage * message = _body_pane->create_followup_or_reply (true);
   if (message) {
-    PostUI * post = PostUI :: create_window (0, _data, _queue, _data, _data, message, _prefs, _group_prefs, _encode_cache);
+    PostUI * post = PostUI :: create_window (nullptr, _data, _queue, _data, _data, message, _prefs, _group_prefs, _encode_cache);
     if (post)
       gtk_widget_show_all (post->root());
     g_object_unref (message);
@@ -1680,7 +1631,7 @@ void GUI :: do_reply_to ()
 void GUI :: do_pan_manual ()
 {
   GError * error (NULL);
-  gtk_show_uri (NULL, "help:pan", gtk_get_current_event_time (), &error);
+  gtk_show_uri_on_window (NULL, "help:pan", gtk_get_current_event_time (), &error);
     if (error) {
       GtkWidget * w = gtk_message_dialog_new (get_window(_root),
                                               GtkDialogFlags(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
@@ -1694,16 +1645,12 @@ void GUI :: do_pan_manual ()
 #endif
 void GUI :: do_pan_web ()
 {
-  URL :: open (_prefs, "http://pan.rebelbase.com/");
+  URL :: open (_prefs, "https://gitlab.gnome.org/GNOME/pan/-/blob/master/README.org");
 }
 void GUI :: do_bug_report ()
 {
   URL :: open (_prefs, "https://gitlab.gnome.org/GNOME/pan/issues");
 }
-void GUI :: do_tip_jar ()
-{
-  URL :: open (_prefs, "http://pan.rebelbase.com/tipjar/");
-}
 void GUI :: do_about_pan ()
 {
   const gchar * authors [] = { "Charles Kerr <charles@rebelbase.com> - Pan Author",
@@ -1712,14 +1659,14 @@ void GUI :: do_about_pan ()
                                "Petr Kovar <pknbe@volny.cz> - Contributor",
                                "Calin Culianu <calin@ajvar.org> - Threaded Decoding",
                                "Christophe Lambin <chris@rebelbase.com> - Original Developer",
-                               "Matt Eagleson <matt@rebelbase.com> - Original Developer", 0 };
-  GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
+                               "Matt Eagleson <matt@rebelbase.com> - Original Developer", nullptr };
+  GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, nullptr);
   GtkAboutDialog * w (GTK_ABOUT_DIALOG (gtk_about_dialog_new ()));
   gtk_about_dialog_set_program_name (w, _("Pan"));
   gtk_about_dialog_set_version (w, PACKAGE_VERSION);
   gtk_about_dialog_set_comments (w, VERSION_TITLE " (" GIT_REV "; " PLATFORM_INFO ")");
   gtk_about_dialog_set_copyright (w, _("Copyright \u00A9 2002-2021 Charles Kerr and others")); // \u00A9 is unicode for (c)
-  gtk_about_dialog_set_website (w, "http://pan.rebelbase.com/");
+  gtk_about_dialog_set_website (w, "https://gitlab.gnome.org/GNOME/pan/-/blob/master/README.org");
   gtk_about_dialog_set_logo (w, logo);
   gtk_about_dialog_set_license (w, LICENSE);
   gtk_about_dialog_set_authors (w, authors);
@@ -1803,7 +1750,7 @@ namespace
 }
 
 void
-GUI :: notebook_page_switched_cb (GtkNotebook *, GtkNotebookPage *, gint page_num, gpointer user_data)
+GUI :: notebook_page_switched_cb (GtkNotebook *, void *, gint page_num, gpointer user_data)
 {
   GUI * gui = static_cast<GUI*>(user_data);
   GtkWidget * w;
@@ -1821,11 +1768,11 @@ void GUI :: do_layout (bool tabbed)
 {
   if (hpane) {
     _prefs.set_int ("main-window-hpane-position", gtk_paned_get_position(GTK_PANED(hpane)));
-    hpane = 0;
+    hpane = nullptr;
   }
   if (vpane) {
     _prefs.set_int ("main-window-vpane-position", gtk_paned_get_position(GTK_PANED(vpane)));
-    vpane = 0;
+    vpane = nullptr;
   }
 
   gtk_widget_hide (_workarea_bin);
@@ -1846,7 +1793,7 @@ void GUI :: do_layout (bool tabbed)
     g_list_free (children);
   }
 
-  GtkWidget * w (0);
+  GtkWidget * w (nullptr);
   if (tabbed)
   {
     w = gtk_notebook_new ();
@@ -1969,7 +1916,7 @@ void GUI :: do_show_selected_article_info ()
 {
   const Article* a = _header_pane->get_first_selected_article ();
 
-  if (a != 0)
+  if (a != nullptr)
   {
     // date
     EvolutionDateMaker date_maker;
@@ -2092,14 +2039,14 @@ void GUI :: do_read_selected_group ()
 
   // if it's the first time in this group, pop up a download-headers dialog.
   // otherwise if get-new-headers is turned on, queue an xover-new task.
-  unsigned long unread(0), total(0);
+  Article_Count unread(0), total(0);
 
   if (changed && !group.empty()){
     _data.get_group_counts (group, unread, total);
     const bool virtual_group (GroupPane::is_virtual_group(group));
     if (!virtual_group)
     {
-      if (!total)
+      if (static_cast<uint64_t>(total) == 0)
         activate_action ("download-headers");
       else if (_prefs.get_flag("get-new-headers-when-entering-group", true)) {
         if (_prefs.get_flag ("mark-group-read-before-xover", false))
@@ -2358,7 +2305,7 @@ GUI :: on_queue_task_active_changed (Queue&, Task& t, bool is_active)
   for (; it!=_active_tasks.end() && i<VIEW_QTY; ++it, ++i)
     _views[i]->set_progress (*it);
   for (; i<VIEW_QTY; ++i)
-    _views[i]->set_progress (0);
+    _views[i]->set_progress (nullptr);
 }
 void
 GUI :: on_queue_connection_count_changed (Queue&, int)
@@ -2423,6 +2370,74 @@ GUI :: on_prefs_string_changed (const StringView& key, const StringView& value)
   }
 }
 
+namespace {
+class Destroyer {
+  public:
+    Destroyer(char *f) :
+      _fname(f)
+    {
+    }
+
+    ~Destroyer()
+    {
+      g_free(_fname);
+    }
+
+    void retain()
+    {
+      _fname = nullptr;
+    }
+
+  private:
+    char *_fname;
+};
+}
+
+void
+GUI :: do_edit_scores (GtkAction *act)
+{
+  //Protect against bouncy keypresses
+  if (not gtk_action_get_sensitive(act)) {
+    return;
+  }
+
+  char *filename{g_strdup(_data.get_scorefile_name().c_str())};
+  Destroyer d{filename};
+  if (not file::file_exists(filename)) {
+    FILE *f = fopen(filename, "a+");
+    if (f == nullptr) {
+      Log::add_err_va("Error creating file '%s'", filename);
+      return;
+    }
+    fclose(f);
+  }
+
+  try
+  {
+    using namespace std::placeholders;
+    _spawner.reset(
+      new EditorSpawner(filename,
+                        std::bind(&GUI::edit_scores_cleanup, this, _1, _2, act),
+                        _prefs));
+    d.retain();
+    gtk_action_set_sensitive(act, false);
+  }
+  catch (EditorSpawnerError const &)
+  {
+    //There should be a big red exclamation on the status line
+  }
+}
+
+void
+GUI :: edit_scores_cleanup(int status, char *filename, GtkAction *act)
+{
+  g_free(filename);
+  _data.rescore();
+  gtk_action_set_sensitive(act, true);
+  _spawner.reset();
+  gtk_window_present(get_window(_root));
+}
+
 #ifdef HAVE_GNUTLS
 
 void
